<?php
namespace BackOfficeOptimizer\Libraries;

use BackOfficeOptimizer\Libraries\JWT;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class JWTChecker extends \Zeedhi\Framework\Events\PreDispatch\Listener {

    const refreshDeadline = 60 * 60; //in seconds
    protected $PermissionProvider;

    public function __construct($PermissionProvider) {
        $this->PermissionProvider = $PermissionProvider;
    }

    /**
     * Listen to request, then throw an exception or
     * continue execution flow according to validity of
     * 'backoffice' token sent through request data.
     *
     * @param Request $request
     */
    public function preDispatch(Request $request) {
        $routePath = $request->getRoutePath();

        if($routePath != '/login' && $routePath != '/importfromreceituariopadrao'){
            $token = isset($request->getParameters()['$token'])? $request->getParameters()['$token']: null;
            
            if(!$token){
                throw new \Exception('FORBIDDEN_ACCESS');       
            } else if(!JWT::validateToken($token)){
                throw new \Exception('INVALID_TOKEN');
            }

            //TODO: move this block to a diff place:
            //refresh token before it's dead
            $lifeEnd = JWT::getTokenHeader($token)['exp'];
            $now = time();
            if($lifeEnd - $now < self::refreshDeadline){
                $payload = JWT::getTokenData($token);
                $permissions = $this->PermissionProvider->getUserPermissions($payload["CDOPERADOR"], $payload["NRORG"]);
                $payload["GRANT_ALL"] = $permissions["grantAll"];
                $payload["SCOPES"] = $permissions["scopes"];
                $newToken = JWT::generateToken($payload);
                header("backofficent: {$newToken}");
                $permissions = json_encode($permissions);
                header("backofficepr: {$permissions}");
            }
        }
    }
}