<?php
namespace Zeedhi\Framework\DBAL;

use Doctrine\Common\EventManager;
use Doctrine\DBAL\Driver;
use Doctrine\DBAL\Configuration;
use Doctrine\DBAL\DriverManager as DoctrineDriverManager;

class DriverManager{
    /**
     * @static getConnection
     *
     * Builds the connection object from the parameters.
     *
     * @param   array          $connection_params    The database connection parameters.
     * @param   Configuration  $config               The doctrine's database config object.
     * @param   EventManager   $eventManager         The doctrine's eventManager object.
     * @param   boolean        $debug_db             The flag that decides the addition or not of
     *                                               the stack trace as comment into the sql string.
     *
     * @return  Connection The connection object.
     */
    public static function getConnection(){
        $arguments = func_get_args();
        $debugDB = array_pop($arguments);
        $connection = DoctrineDriverManager::getConnection(...$arguments);
        $params = $connection->getParams();
        $config = $connection->getConfiguration();
        $driver = $connection->getDriver();
        $eventManager = $connection->getEventManager();
        return new Connection($params, $driver, $config, $eventManager, $debugDB);
    }
}